#include "glew.h"
#include "freeglut.h"
#include "glm.hpp"
#include "ext.hpp"
#include <iostream>
#include <cmath>

#include "Shader_Loader.h"
#include "Render_Utils.h"
#include "Camera.h"
#include "Texture.h"

GLuint programTexture;
GLuint programColor;

GLuint texture;


Core::Shader_Loader shaderLoader;

obj::Model FishModel;
obj::Model sphereModel;

float cameraAngle = 0;
glm::vec3 cameraPos = glm::vec3(0.5, 0, 0.5);
glm::vec3 cameraDir;

glm::vec3 lightDir = glm::normalize(glm::vec3(1.0f, -0.9f, -1.0f));

glm::mat4 cameraMatrix, perspectiveMatrix;


//define fish particle structure
//...

std::vector<float> tangent(1203);
std::vector<float> bitangent(1203);

float moveSpeed = 0.1;
int windowH = 300;
int windowW = 300;
float roll = 0.0;
float yaw = 0.0;
float pitch = 0.0;
glm::mat4 viewMatrix;
glm::mat4 shipRotateMatrix = glm::mat4(1.0);
glm::quat camera_quat;



void keyboard(unsigned char key, int x, int y)
{
	float angleSpeed = 0.1f;
	switch(key)
	{
	case 'z': cameraAngle -= angleSpeed; break;
	case 'x': cameraAngle += angleSpeed; break;
	case 'w': cameraPos += cameraDir * moveSpeed; break;
	case 's': cameraPos -= cameraDir * moveSpeed; break;
	case 'd': cameraPos += glm::cross(cameraDir, glm::vec3(0, 1, 0)) * moveSpeed; break;
	case 'a': cameraPos -= glm::cross(cameraDir, glm::vec3(0, 1, 0)) * moveSpeed; break;

	}

}



void reshape(int w, int h)
{

	windowH = h;
	windowW = w;


	glViewport(0, 0, w, h);

	glutPostRedisplay();
}



glm::mat4 createCameraMatrixQ()
{
	cameraDir = glm::vec3(cosf(cameraAngle), 0.0f, sinf(cameraAngle));

	glm::quat key_quat = glm::quat(glm::vec3(pitch, yaw, roll));

	pitch = yaw = roll = 0;


	camera_quat = key_quat * camera_quat;
	camera_quat = glm::normalize(camera_quat);
	glm::mat4 rotate = glm::mat4_cast(camera_quat);

	glm::mat4 translate = glm::mat4(1.0f);
	translate = glm::translate(translate, -cameraPos);

	viewMatrix = rotate * translate;

	return viewMatrix;

}

void drawObjectColor(obj::Model * model, glm::mat4 modelMatrix, glm::vec3 color)
{
	GLuint program = programColor;

	glUseProgram(program);

	glUniform3f(glGetUniformLocation(program, "objectColor"), color.x, color.y, color.z);
	glUniform3f(glGetUniformLocation(program, "lightDir"), lightDir.x, lightDir.y, lightDir.z);

	glm::mat4 transformation = perspectiveMatrix * cameraMatrix * modelMatrix;
	glUniformMatrix4fv(glGetUniformLocation(program, "modelViewProjectionMatrix"), 1, GL_FALSE, (float*)&transformation);
	glUniformMatrix4fv(glGetUniformLocation(program, "modelMatrix"), 1, GL_FALSE, (float*)&modelMatrix);

	Core::DrawModel(model);

	glUseProgram(0);
}

void drawObjectTexture(obj::Model * model, glm::mat4 modelMatrix, GLuint texture)
{
	GLuint program = programTexture;

	glUseProgram(program);

	Core::SetActiveTexture(texture, "colorTexture", program, 0);	
	glUniform3f(glGetUniformLocation(program, "lightDir"), lightDir.x, lightDir.y, lightDir.z);

	glm::mat4 transformation = perspectiveMatrix * cameraMatrix * modelMatrix;
	glUniformMatrix4fv(glGetUniformLocation(program, "modelViewProjectionMatrix"), 1, GL_FALSE, (float*)&transformation);
	glUniformMatrix4fv(glGetUniformLocation(program, "modelMatrix"), 1, GL_FALSE, (float*)&modelMatrix);

	Core::DrawModel(model);

	glUseProgram(0);
}


void renderScene()
{
	float time = glutGet(GLUT_ELAPSED_TIME) / 10.0f;


	cameraMatrix = createCameraMatrixQ();

	perspectiveMatrix = Core::createPerspectiveMatrix(0.1f, 100.0f, windowH, windowW);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

	glm::mat4 FishModelMatrix;

	//draw fish particles
	//...
	
	
	glutSwapBuffers();
}

void init()
{
	glEnable(GL_DEPTH_TEST);
	programTexture = shaderLoader.CreateProgram("shaders/shader_tex.vert", "shaders/shader_tex.frag");
	programColor = shaderLoader.CreateProgram("shaders/shader_color.vert", "shaders/shader_color.frag");
	

	texture = Core::LoadTexture("textures/fish_texture.png");


	sphereModel = obj::loadModelFromFile("models/sphere.obj");
	FishModel = obj::loadModelFromFile("models/sphere.obj");

	//create fish particles
	//...
	

	std::vector<float> unitY = {0.0, 1.0, 0.0};
	std::vector<float> unitX = { 1.0, 0.0, 0.0 };
	glm::vec3 uY = { 0.0, -1.0, 0.0 };
	glm::vec3 uX = { -1.0, 0.0, 0.0 };
	glm::vec3 tang[1203];
	glm::vec3 bitang[1203];

	for (int i = 0; i < sphereModel.normal.size(); i += 3)
	{
		glm::vec3 normal = { sphereModel.normal[i + 0], sphereModel.normal[i + 1], sphereModel.normal[i + 2] };
		if (sphereModel.normal[i + 1] < 0.99 && sphereModel.normal[i + 1] > -0.99) tang[i] = glm::normalize(glm::cross(normal, uY));
		else tang[i] = glm::normalize(glm::cross(normal, uX));
		bitang[i] = glm::normalize(glm::cross(normal, tang[i]));
		tangent[i + 0] = tang[i].x;
		tangent[i + 1] = tang[i].y;
		tangent[i + 2] = tang[i].z;
	}
	
}

void shutdown()
{
	shaderLoader.DeleteProgram(programTexture);
	shaderLoader.DeleteProgram(programColor);
}

void idle()
{
	glutPostRedisplay();
}




int main(int argc, char ** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(200, 200);
	windowH = 600;
	windowW = 600;
	glutInitWindowSize(windowH, windowW);
	glutCreateWindow("CGP");
	glewInit();

	init();
	glutKeyboardFunc(keyboard);
	glutReshapeFunc(reshape);
	glutDisplayFunc(renderScene);
	glutIdleFunc(idle);
	
	glutMainLoop();

	shutdown();

	return 0;
}
